﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

namespace WcfRestaurant.Dalc
{
    public class dalcLibreriaBD
    {
        //private MySql.Data.MySqlClient.MySqlConnection conBaseDatos;
        private System.Data.SqlClient.SqlConnection conBaseDatos;

        ///<summary>
        ///Abre la conexión con la base de datos definida.
        ///</summary>
        public void Conectar()
        {
            //string CadenaConexion = "Driver={SQL Server Native Client 10.0};Server=(local);Database=Restaurant;Uid=dani;Pwd=fibla;";
            string CadenaConexion = "Server=(local);Database=Restaurant;Uid=dani;Pwd=fibla;";
            conBaseDatos = new System.Data.SqlClient.SqlConnection(CadenaConexion);
            conBaseDatos.Open();
        }

        ///<summary>
        ///Cierra la conexión con la base de datos en caso de que este abierta.
        ///</summary>
        public void Desconectar()
        {
            if (conBaseDatos != null)
            {
                if (conBaseDatos.State == ConnectionState.Open) 
                {
                    conBaseDatos.Close();
                }
            }
        }

        ///<summary>
        ///Ejecuta una sentencia SQL que no requiera la obtención de datos. 
        ///</summary>
        ///<return>
        ///Devuelve una cadena de texto con el resultado de la ejecución de la sentencia:
        ///     - Devuelve cadena en blanco si no hay errores en la ejecución de la sentencia.
        ///     - Devuelve el mensaje de excepción en caso de errores en la ejecución de la sentencia.
        ///</return>
        ///<param name="strSQL">
        ///Cadena con la sentencia SQL a ejecutar en la base de datos.
        ///</param>
        //public string EjecutarSentencia(string strSQL, MySql.Data.MySqlClient.MySqlParameter[] mysqlParametros )
        public string EjecutarSentencia(string strSQL, System.Data.SqlClient.SqlParameter[] mysqlParametros)
        {
            string strRet = "";
            int intRet = 0;
            try
            {
                Conectar();
                ///MySql.Data.MySqlClient.MySqlCommand cmdSQL = new MySql.Data.MySqlClient.MySqlCommand();
                System.Data.SqlClient.SqlCommand cmdSQL = new System.Data.SqlClient.SqlCommand();
                cmdSQL.Connection = conBaseDatos;
                cmdSQL.CommandType = CommandType.Text; ;
                cmdSQL.CommandText = strSQL;
                if (mysqlParametros != null)
                {
                    if (mysqlParametros.Length > 0)
                    {
                        foreach (System.Data.SqlClient.SqlParameter mySqlParam in mysqlParametros)
                        {
                            cmdSQL.Parameters.Add(mySqlParam);
                        }
                    }
                }
                intRet = System.Convert.ToInt32(cmdSQL.ExecuteScalar());
            }
            catch (Exception ex)
            {
                strRet = ex.Message.ToString();
            }
            finally
            {
                Desconectar();
            }
            return strRet;
        }

        ///<summary>
        ///Ejecuta una sentencia SQL que consulte o obtenga valores de la base de datos. 
        ///</summary>
        ///<return>
        ///Devuelve un DataSet con los valores que cumplen las condiciones de la sentencia SQL ejecutada.
        ///</return>
        ///<param name="strSQL">
        ///Cadena con la sentencia SQL a ejecutar en la base de datos.
        ///</param>
        public DataSet GenerarSelect(string strSQL)
        {
            string strRet = "";
            int intRet = 0;
            DataSet dsDatos = new DataSet(); 
            try
            {
                Conectar();
                //MySql.Data.MySqlClient.MySqlDataAdapter sqlAdapterMySQL = new MySql.Data.MySqlClient.MySqlDataAdapter(strSQL, conBaseDatos);
                System.Data.SqlClient.SqlDataAdapter sqlAdapterMySQL = new System.Data.SqlClient.SqlDataAdapter(strSQL, conBaseDatos);
                intRet = sqlAdapterMySQL.Fill(dsDatos);
            }
            catch (Exception ex)
            {
                strRet = ex.Message.ToString();
            }
            finally
            {
                Desconectar();
            }
            return dsDatos;
        }

        ///<summary>
        ///Ejecuta una sentencia SQL para consultar cuantos registros existen que cumplan dicha sentencia. 
        ///</summary>
        ///<return>
        ///Devuelve el numero de registros que complen la sentencia SQL.
        ///</return>
        ///<param name="strSQL">
        ///Cadena con la sentencia SQL a ejecutar en la base de datos.
        ///</param>
        public int ContarRegistrosSelect(string strSQL)
        {
            int intRet = 0;
            try
            {
                Conectar();
                //MySql.Data.MySqlClient.MySqlCommand cmdSQL = new MySql.Data.MySqlClient.MySqlCommand();
                System.Data.SqlClient.SqlCommand cmdSQL = new System.Data.SqlClient.SqlCommand();
                cmdSQL.Connection = conBaseDatos;
                cmdSQL.CommandType = CommandType.Text; ;
                cmdSQL.CommandText = strSQL;

                intRet = System.Convert.ToInt32(cmdSQL.ExecuteScalar());
            }
            catch (Exception ex)
            {
                intRet = -1;
                Console.WriteLine(ex.Message.ToString());
            }
            finally
            {
                Desconectar();
            }
            return intRet;
        }

        ///<summary>
        ///Aplica un formato correcto a los valores para ser contenidos en una sentencia SQL.
        ///</summary>
        ///<return>
        ///Devuelve una cadena de texto con el valor preparado para ser incluido en una sentencia SQL.
        ///</return>
        ///<param name="strValor">
        ///Cadena de texto con el valor que hay que transformar a cadena de SQL.
        ///</param>
        public string VtoSQL(string strValor)
        {
            if (strValor.Trim().Length > 0)
            {
                return "'" + strValor + "'";
            }
            else
            {
                return "null";
            }
        }

        ///<summary>
        ///Aplica un formato correcto a los valores para ser contenidos en una sentencia SQL.
        ///</summary>
        ///<return>
        ///Devuelve una cadena de texto con el valor preparado para ser incluido en una sentencia SQL.
        ///</return>
        ///<param name="strValor">
        ///DateTime con el valor que hay que transformar a cadena de SQL.
        ///</param>
        public string VtoSQL(DateTime? dtValor)
        {
            if (dtValor != null)
            {
                return "'" + String.Format("{0:yyyy-MM-dd}", dtValor) + "'";
            }
            else
            {
                return "null";
            }
        }

        ///<summary>
        ///Aplica un formato correcto a los valores para ser contenidos en una sentencia SQL.
        ///</summary>
        ///<return>
        ///Devuelve una cadena de texto con el valor preparado para ser incluido en una sentencia SQL.
        ///</return>
        ///<param name="intValor">
        ///Entero con el valor que hay que transformar a cadena de SQL.
        ///</param>
        public string VtoSQL(int intValor)
        {
            return "" + intValor + "";
        }

        ///<summary>
        ///Convierte los valores enviados a un objeto parametro para ejecutar las sentencias SQL.
        ///</summary>
        ///<return>
        ///Devuelve un objeto parametro que contiene los valores para ejecutar la sentencia SQL.
        ///</return>
        ///<param name="idCampoSQL">
        ///Nombre del campo que se identifica en la sentencia SQL.
        ///</param>
        ///<param name="strValor">
        ///Cadena con el contenido del valor a informar en el identificador de la sentencia.
        ///</param>
        /// <param name="puedeSerNulo">
        /// Indicador boleano que define si el parametro passado puede ser nulo.
        ///</param>
        //public MySql.Data.MySqlClient.MySqlParameter CrearParametroSQL(string idCampoSQL, string strValor, bool puedeSerNulo)
        public System.Data.SqlClient.SqlParameter CrearParametroSQL(string idCampoSQL, string strValor, bool puedeSerNulo)
        {
            System.Data.SqlClient.SqlParameter paramVariable = new System.Data.SqlClient.SqlParameter();
            paramVariable.ParameterName = idCampoSQL;
            //paramVariable.MySqlDbType = MySql.Data.MySqlClient.MySqlDbType.VarChar;
            paramVariable.SqlDbType = System.Data.SqlDbType.VarChar;
            paramVariable.IsNullable = puedeSerNulo;
            paramVariable.Value = strValor;

            return paramVariable;
        }

        ///<summary>
        ///Convierte los valores enviados a un objeto parametro para ejecutar las sentencias SQL.
        ///</summary>
        ///<return>
        ///Devuelve un objeto parametro que contiene los valores para ejecutar la sentencia SQL.
        ///</return>
        ///<param name="idCampoSQL">
        ///Nombre del campo que se identifica en la sentencia SQL.
        ///</param>
        ///<param name="dtValor">
        ///DateTime con el contenido del valor a informar en el identificador de la sentencia.
        ///</param>
        /// <param name="puedeSerNulo">
        /// Indicador boleano que define si el parametro passado puede ser nulo.
        ///</param>
        //public MySql.Data.MySqlClient.MySqlParameter CrearParametroSQL(string idCampoSQL, DateTime? dtValor, bool puedeSerNulo)
        public System.Data.SqlClient.SqlParameter CrearParametroSQL(string idCampoSQL, DateTime? dtValor, bool puedeSerNulo)
        {
            System.Data.SqlClient.SqlParameter paramVariable = new System.Data.SqlClient.SqlParameter();
            paramVariable.ParameterName = idCampoSQL;
            paramVariable.SqlDbType = System.Data.SqlDbType.DateTime;
            paramVariable.IsNullable = puedeSerNulo;
            paramVariable.Value = dtValor;

            return paramVariable;
        }


        ///<summary>
        ///Convierte los valores enviados a un objeto parametro para ejecutar las sentencias SQL.
        ///</summary>
        ///<return>
        ///Devuelve un objeto parametro que contiene los valores para ejecutar la sentencia SQL.
        ///</return>
        ///<param name="idCampoSQL">
        ///Nombre del campo que se identifica en la sentencia SQL.
        ///</param>
        ///<param name="intValor">
        ///Numero entero con el contenido del valor a informar en el identificador de la sentencia.
        ///</param>
        /// <param name="puedeSerNulo">
        /// Indicador boleano que define si el parametro passado puede ser nulo.
        ///</param>
        //public MySql.Data.MySqlClient.MySqlParameter CrearParametroSQL(string idCampoSQL, int intValor, bool puedeSerNulo)
        public System.Data.SqlClient.SqlParameter CrearParametroSQL(string idCampoSQL, int intValor, bool puedeSerNulo)
        {
            System.Data.SqlClient.SqlParameter paramVariable = new System.Data.SqlClient.SqlParameter();
            paramVariable.ParameterName = idCampoSQL;
            paramVariable.SqlDbType = System.Data.SqlDbType.Int;
            paramVariable.IsNullable = puedeSerNulo;
            paramVariable.Value = intValor;

            return paramVariable;
        }
    }
}
