﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using WcfRestaurant.Procesos;


namespace WcfRestaurant.Dalc
{
    public class dalcLiniesComada
    {
         dalcLlibreriaBD objBD;

        ///<summary>
        ///Constructor de l'objecte que gestiona l'acces a dades.
        ///</summary>
         public dalcLiniesComada()
        {
            objBD = new dalcLlibreriaBD();
        }

        ///<summary>
        ///Obté les línies d'una comanda en concret de la base de dades i informa les seves dades en un objecte Comanda.
        ///</summary>
        ///<return>
         ///Retorna un objecte LineaComanda amb les dades del registre amb l'identificador solicitat.
        ///</return>
         ///<param name="IdComanda">
        ///Cadena amb l'identificador de registre d'una comanda.
        ///</param>
         public List<ComuRestaurant.Objectes.LineaComanda> ObtindreLiniesPerComanda(string IdComanda)
        {
            string strSQL = "";
            ComuRestaurant.Objectes.LineaComanda objLinea = null;
            ComuRestaurant.Objectes.LiniesComanda colLinies = new ComuRestaurant.Objectes.LiniesComanda();

            strSQL = "SELECT idLineaComanda,IdComanda,IdPlat1,IdPlat2,IdPostre FROM LiniesComanda Where IdComanda=" +  objBD.VtoSQL(IdComanda)  + " AND Servida=0";
            DataSet ds = objBD.GenerarSelect(strSQL);
            for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
            {
                dalcPlats dalcPlat = new dalcPlats();
                objLinea = new ComuRestaurant.Objectes.LineaComanda();
                objLinea.IdLineaComanda = Convert.ToInt32(ds.Tables[0].Rows[i]["idLineaComanda"]);
                objLinea.Comanda = Convert.ToInt32(ds.Tables[0].Rows[i]["IdComanda"]); 
                objLinea.Plat1 = dalcPlat.Obtindre(ds.Tables[0].Rows[i]["IdPlat1"].ToString());
                objLinea.Plat2 = dalcPlat.Obtindre(ds.Tables[0].Rows[i]["IdPlat2"].ToString());
                objLinea.Postre = dalcPlat.Obtindre(ds.Tables[0].Rows[i]["IdPostre"].ToString());

                colLinies.addLineaComada(objLinea);
            }
            return colLinies.getllista(); 
        }
        

        //<summary>
        ///Obté una llista de les linies de comanda que existeixen a la base de dades i informa les seves dades d'un objecte LiniesComanda.
        ///</summary>
        ///<return>
        ///Retornat un objecte comanda que conte una llista d'objetes comanda amb la informació de les camandes.
        ///</return>
        public List<ComuRestaurant.Objectes.LineaComanda> ObtindreTotes()
        {
            string strSQL = "";
            ComuRestaurant.Objectes.LineaComanda objLinea = null;
            ComuRestaurant.Objectes.LiniesComanda colLinies = new ComuRestaurant.Objectes.LiniesComanda();

            strSQL = "SELECT idLineaComanda,IdPlat1,IdPlat2,IdPostre FROM LiniesComanda Where Servida=0";
            DataSet ds = objBD.GenerarSelect(strSQL);
            for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
            {
                dalcPlats dalcPlat = new dalcPlats();
                objLinea = new ComuRestaurant.Objectes.LineaComanda();
                objLinea.IdLineaComanda = Convert.ToInt32(ds.Tables[0].Rows[i]["idLineaComanda"]);
                objLinea.Plat1 = dalcPlat.Obtindre(ds.Tables[0].Rows[i]["IdPlat1"].ToString());
                objLinea.Plat2 = dalcPlat.Obtindre(ds.Tables[0].Rows[i]["IdPlat2"].ToString());
                objLinea.Postre = dalcPlat.Obtindre(ds.Tables[0].Rows[i]["IdPostre"].ToString());
                
                colLinies.addLineaComada(objLinea);
            }
            return colLinies.getllista(); 
        }

        /// <summary>
        /// Obté el total de linies de Comanda que te una comanda en concret a la base de dades.
        /// </summary>
        /// <returns>
        /// Retorna un integer amb el número total de linies que te una comanda a la base de dades.
        /// </returns>
        public int TotalLiniesComandesPerComanda(string IdComanda)
        {
            string strSQL = "SELECT Count(*) As Total FROM LiniesComanda WHERE IdComanda=" + objBD.VtoSQL(IdComanda);
            DataSet ds = objBD.GenerarSelect(strSQL);
            return Convert.ToInt32(ds.Tables[0].Rows[0]["Total"]);
            //return ds.Tables[0].Rows.Count ;
        }


        ///<summary>
        ///Elimina una linea de comanda  en concret de la base de dades.
        ///</summary>
        ///<return>
        ///retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="IdLinea">
        ///Cadena amb l'identificador de registre d'una linea de comanda.
        ///</param>
        public string EliminarLineaComanda(string IdLinea)
        {

            string strSQL = "DELETE FROM LiniesComanda WHERE IdLineaComanda=@IdLinea";
            System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[1];
            sSqlParametros[0] = objBD.CrearParametreSQL("@IdLinea", IdLinea, false);
            string strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);

            return strRet;
        }


        /// <summary>
        /// Actualitza l'estat de la línea de comanda un cop es dona per servida.
        /// </summary>
        /// <returns>
        /// /retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        /// </returns>
        ///<param name="IdLinea">
        ///Cadena amb l'identificador de registre d'una linea de comanda.
        ///</param>
        public string LineaServida(string IdLinea)
        {
            string strSQL = "UPDATE LiniesComanda SET Servida=1 WHERE IdLineaComanda=@IdLineaComanda";
            System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[1];
            sSqlParametros[0] = objBD.CrearParametreSQL("@IdLineaComanda", IdLinea, false);
            string strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);
            return strRet;
        }

        /// <summary>
        /// Obté el total de linies de Comanda que te una comanda en concret a la base de dades.
        /// </summary>
        /// <returns>
        /// Retorna un integer amb el número total de linies que te una comanda a la base de dades.
        /// </returns>
        public int TotalLiniesComandes()
        {
            string strSQL = "SELECT Count(*) FROM LiniesComanda WHERE Servida=0";
            DataSet ds = objBD.GenerarSelect(strSQL);
            return ds.Tables[0].Rows.Count;
        }


        /// <summary>
        /// Obté el total de linies de Comanda que te una comanda en concret a la base de dades.
        /// </summary>
        /// <returns>
        /// Retorna un integer amb el número total de linies que te una comanda a la base de dades.
        /// </returns>
        ///<param name="IdComanda">
        ///Cadena amb l'identificador de registre d'una linea de comanda.
        ///</param>
        ///<param name="IdPlat1">
        ///Cadena amb l'identificador del Primer plat de la linea de comanda.
        ///</param>
        ///<param name="IdPlat2">
        ///Cadena amb l'identificador del segon plat de la linea de comanda.
        ///</param>
        ///<param name="IdPostre">
        ///Cadena amb l'identificador del postre de la linea de comanda.
        ///</param>
        public string DesarLineaComanda(string IdComanda,string IdPlat1, string IdPlat2, string IdPostre) {

            string strSQL = "INSERT INTO LiniesComanda(IdComanda,IdPlat1,IdPlat2,IdPostre,Servida) VALUES(@IdComanda,@IdPlat1,@IdPlat2,@IdPostre,0)";
            System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[4];
            sSqlParametros[0] = objBD.CrearParametreSQL("@IdComanda", IdComanda, false);
            sSqlParametros[1] = objBD.CrearParametreSQL("@IdPlat1", IdPlat1, false);
            sSqlParametros[2] = objBD.CrearParametreSQL("@IdPlat2", IdPlat2, false);
            sSqlParametros[3] = objBD.CrearParametreSQL("@IdPostre", IdPostre, false);
            string strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);
            return strRet;
        }
    }
}
