﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

namespace WcfRestaurant.Dalc
{
    public class dalcLlibreriaBD
    {
        private System.Data.SqlClient.SqlConnection conBaseDades;

        ///<summary>
        ///Obre la connexió amb la Base de dades definida
        ///</summary>
        public void Connectar()
        {
            string CadenaConnexio = "Server=(local);Database=Restaurant;Uid=dani;Pwd=dani;";
            conBaseDades = new System.Data.SqlClient.SqlConnection(CadenaConnexio);
            conBaseDades.Open();
        }

        ///<summary>
        ///Tanca la connexió amb la base de dades en cas de que esigui oberta
        ///</summary>
        public void Desconnectar()
        {
            if (conBaseDades != null)
            {
                if (conBaseDades.State == ConnectionState.Open) 
                {
                    conBaseDades.Close();
                }
            }
        }

        ///<summary>
        ///Executa una sentencia SQL que no requereixi l'obtenció de dades
        ///</summary>
        ///<return>
        ///Retorna una cadana de text amb el resultat de l'execució de la sentència
        ///     - Retorna una cadena en blanc si no hi ha errors en l'execució de la sentència
        ///     - Retorna el missatge d'excepció en cas d'errors a l'execució de la sentència.
        ///</return>
        ///<param name="strSQL">
        ///Cadena amb la sentència SQL a executar en la base de dades.
        ///</param>
        public string ExecutarSentencia(string strSQL, System.Data.SqlClient.SqlParameter[] sqlParametres)
        {
            string strRet = "";
            int intRet = 0;
            try
            {
                Connectar();
                System.Data.SqlClient.SqlCommand cmdSQL = new System.Data.SqlClient.SqlCommand();
                cmdSQL.Connection = conBaseDades;
                cmdSQL.CommandType = CommandType.Text; ;
                cmdSQL.CommandText = strSQL;
                if (sqlParametres != null)
                {
                    if (sqlParametres.Length > 0)
                    {
                        foreach (System.Data.SqlClient.SqlParameter sSqlParam in sqlParametres)
                        {
                            cmdSQL.Parameters.Add(sSqlParam);
                        }
                    }
                }
                intRet = System.Convert.ToInt32(cmdSQL.ExecuteScalar());
            }
            catch (Exception ex)
            {
                strRet = ex.Message.ToString();
            }
            finally
            {
                Desconnectar();
            }
            return strRet;
        }

        ///<summary>
        ///Executa una sentència SQL que consulta o obté valore de la base de dades. 
        ///</summary>
        ///<return>
        ///Retorna un DataSet amb els valors que cumpleixen les condicions de la sentència SQL executada.
        ///</return>
        ///<param name="strSQL">
        ///Cadena amb la sentència SQL a executar en la base de dades.
        ///</param>
        public DataSet GenerarSelect(string strSQL)
        {
            string strRet = "";
            int intRet = 0;
            DataSet dsDatos = new DataSet(); 
            try
            {
                Connectar();
                System.Data.SqlClient.SqlDataAdapter sqlAdapterSQL = new System.Data.SqlClient.SqlDataAdapter(strSQL, conBaseDades);
                intRet = sqlAdapterSQL.Fill(dsDatos);
            }
            catch (Exception ex)
            {
                strRet = ex.Message.ToString();
            }
            finally
            {
                Desconnectar();
            }
            return dsDatos;
        }

        ///<summary>
        ///Executa una sentència SQL per a consultar qunats registres existeixen que cumpleixin la sentència. 
        ///</summary>
        ///<return>
        ///Retorna el numero de registres que cumpleixen la sentència SQL.
        ///</return>
        ///<param name="strSQL">
        ///Cadena amb la sentència SQL a executar a la base de dades.
        ///</param>
        public int ComptarRegistresSelect(string strSQL)
        {
            int intRet = 0;
            try
            {
                Connectar();
                System.Data.SqlClient.SqlCommand cmdSQL = new System.Data.SqlClient.SqlCommand();
                cmdSQL.Connection = conBaseDades;
                cmdSQL.CommandType = CommandType.Text; ;
                cmdSQL.CommandText = strSQL;

                intRet = System.Convert.ToInt32(cmdSQL.ExecuteScalar());
            }
            catch (Exception ex)
            {
                intRet = -1;
                Console.WriteLine(ex.Message.ToString());
            }
            finally
            {
                Desconnectar();
            }
            return intRet;
        }

        ///<summary>
        ///Aplica un format correcte als valors per a ser continguts en una sentència SQL.
        ///</summary>
        ///<return>
        ///Retorna una cadena de texte amb el valor preparat per a ser inclós en una sentència SQL.
        ///</return>
        ///<param name="strValor">
        ///Cadena de texte amb el valor que hi ha que transformar a cadena de SQL.
        ///</param>
        public string VtoSQL(string strValor)
        {
            if (strValor.Trim().Length > 0)
            {
                return "'" + strValor + "'";
            }
            else
            {
                return "null";
            }
        }

        ///<summary>
        ///Aplica un format correcte als valors per a ser continguts en una sentència SQL.
        ///</summary>
        ///<return>
        ///Retorna una cadena de texte amb el valor preparat per a ser inclós en una sentència SQL.
        ///</return>
        ///<param name="strValor">
        ///DateTime amb el valor que hi ha que transformar a cadena de SQL.
        ///</param>
        public string VtoSQL(DateTime? dtValor)
        {
            if (dtValor != null)
            {
                return "'" + String.Format("{0:yyyy-MM-dd}", dtValor) + "'";
            }
            else
            {
                return "null";
            }
        }

        ///<summary>
        ///Aplica un format correcte als valors per a ser continguts en una sentència SQL.
        ///</summary>
        ///<return>
        ///Retorna una cadena de texte amb el valor preparat per a ser inclós en una sentència SQL.
        ///</return>
        ///<param name="intValor">
        ///Sencer amb el valor que hi ha que transformar a cadena de SQL.
        ///</param>
        public string VtoSQL(int intValor)
        {
            return "" + intValor + "";
        }

        ///<summary>
        ///Converteix els valores enviats a un objecte paràmetre per executar les sentències SQL.
        ///</summary>
        ///<return>
        ///Retorna un objecte paràmetre que conté els valors per executar la sentència SQL.
        ///</return>
        ///<param name="idCampSQL">
        ///Nom del camp que s'identifica a la sentència SQL.
        ///</param>
        ///<param name="strValor">
        ///Cadena amb el contingut del valor a informar a l'identificador de la sentència.
        ///</param>
        /// <param name="potSerNul">
        ///Indicador booleà que defineix si el paràmetre passat pot ser nul.
        ///</param>
        public System.Data.SqlClient.SqlParameter CrearParametreSQL(string idCampSQL, string strValor, bool potSerNul)
        {
            System.Data.SqlClient.SqlParameter paramVariable = new System.Data.SqlClient.SqlParameter();
            paramVariable.ParameterName = idCampSQL;
            paramVariable.SqlDbType = System.Data.SqlDbType.VarChar;
            paramVariable.IsNullable = potSerNul;
            paramVariable.Value = strValor;

            return paramVariable;
        }

        ///<summary>
        ///Converteix els valores enviats a un objecte paràmetre per executar les sentències SQL.
        ///</summary>
        ///<return>
        ///Retorna un objecte paràmetre que conté els valors per executar la sentència SQL.
        ///</return>
        ///<param name="idCampSQL">
        ///Nom del camp que s'identifica a la sentència SQL.
        ///</param>
        ///<param name="dtValor">
        ///DateTime amb el contingut del valor a informar a l'identificador de la sentència.
        ///</param>
        /// <param name="potSerNul">
        ///Indicador booleà que defineix si el paràmetre passat pot ser nul.
        ///</param>
        public System.Data.SqlClient.SqlParameter CrearParametreSQL(string idCampSQL, DateTime? dtValor, bool potSerNul)
        {
            System.Data.SqlClient.SqlParameter paramVariable = new System.Data.SqlClient.SqlParameter();
            paramVariable.ParameterName = idCampSQL;
            paramVariable.SqlDbType = System.Data.SqlDbType.DateTime;
            paramVariable.IsNullable = potSerNul;
            paramVariable.Value = dtValor;

            return paramVariable;
        }


        ///<summary>
        ///Converteix els valores enviats a un objecte paràmetre per executar les sentències SQL.
        ///</summary>
        ///<return>
        ///Retorna un objecte paràmetre que conté els valors per executar la sentència SQL.
        ///</return>
        ///<param name="idCampSQL">
        ///Nom del camp que s'identifica a la sentència SQL
        ///</param>
        ///<param name="intValor">
        ///Numero sencer amb el contingut del valor a informar a l'identificador de la sentència.
        ///</param>
        /// <param name="potSerNul">
        ///Indicador booleà que defineix si el paràmetre passat pot ser nul.
        ///</param>
        public System.Data.SqlClient.SqlParameter CrearParametreSQL(string idCampSQL, int intValor, bool potSerNul)
        {
            System.Data.SqlClient.SqlParameter paramVariable = new System.Data.SqlClient.SqlParameter();
            paramVariable.ParameterName = idCampSQL;
            paramVariable.SqlDbType = System.Data.SqlDbType.Int;
            paramVariable.IsNullable = potSerNul;
            paramVariable.Value = intValor;

            return paramVariable;
        }
    }
}
