﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using WcfRestaurant.Procesos;

namespace WcfRestaurant.Dalc
{
    public class dalcMenu
    {
        dalcLlibreriaBD objBD;

        ///<summary>
        ///Constructor de l'objecte que gestiona l'acces a dades.
        ///</summary>
        public dalcMenu()
        {
            objBD = new dalcLlibreriaBD();
        }

        ///<summary>
        ///Guarda la informació de l'objecte menu a la base de dades. Ja sigui insertant un registre nou o actualizat un registre ja existent.
        ///</summary>
        ///<return>
        ///Retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="objMenu">
        ///Objecte menu que conté la informació que es vol guardar.
        ///</param>
        public string Desar(ComuRestaurant.Objectes.Menu objMenu)
        {
            string strRet = "";
            if (objMenu!=null){
                if (objMenu.Nom.Trim().Length > 0)
                {
                    string strSQL = "";
                    bool bExisteix = Existeix(objMenu.IdMenu);

                    dalcPlats dalcPlat = new dalcPlats();

                    System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[9];
                    sSqlParametros[0] = objBD.CrearParametreSQL("@Nom", objMenu.Nom, false);
                    sSqlParametros[1] = objBD.CrearParametreSQL("@IdPlat1",dalcPlat.ObtenirIdentificador(objMenu.Plat1.NomCAT), true);
                    sSqlParametros[2] = objBD.CrearParametreSQL("@IdPlat2", dalcPlat.ObtenirIdentificador(objMenu.Plat2.NomCAT), true);
                    sSqlParametros[3] = objBD.CrearParametreSQL("@IdPostre", dalcPlat.ObtenirIdentificador(objMenu.Postre.NomCAT), true);
                    sSqlParametros[4] = objBD.CrearParametreSQL("@Comensals", objMenu.Comensals, false);
                    sSqlParametros[5] = objBD.CrearParametreSQL("@IVA", Convert.ToString(objMenu.IVA), false);
                    sSqlParametros[6] = objBD.CrearParametreSQL("@Preu",objMenu.Preu, false);
                    sSqlParametros[7] = objBD.CrearParametreSQL("@Beguda",Convert.ToString(objMenu.Beguda), false);
                    sSqlParametros[8] = objBD.CrearParametreSQL("@IdMenu", Convert.ToString(objMenu.IdMenu), false);
               
                    if (bExisteix){
                        strSQL = "UPDATE Menus SET Nom = @Nom,IdPlat1=@IdPlat1,IdPlat2=@IdPlat2,IdPostre=@IdPostre,Comensals=@Comensals,IVA=@IVA,Preu=@Preu,Beguda=@Beguda WHERE IdMenu=@IdMenu";
                    }else {
                        strSQL = "INSERT INTO Menus (Nom,IdPlat1,IdPlat2,IdPostre,Comensals,IVA,Preu,Beguda) VALUES (@Nom,@IdPlat1,@IdPlat2,@IdPostre,@Comensals,@IVA,@Preu,@Beguda)";
                    }
                    strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);
                }else{
                    strRet = "El nom de l'objecte menú no esta informat.";
                }
            }else{
                strRet = "L'objecte del menú esta vuit.";
            }
            return  strRet;
        }

        ///<summary>
        ///Obté un menu en concret de la base de dades i informa les seves dades en un objecte menu.
        ///</summary>
        ///<return>
        ///Rttorna un objecte Menu amb les dades del registre amb l'identificador solicitat.
        ///</return>
        ///<param name="IdMenu">
        ///Cadena amb l'identificador del registre d'un menu.
        ///</param>
        public ComuRestaurant.Objectes.Menu Obtindre(string IdMenu)
        {
            string strSQL = "";
            ComuRestaurant.Objectes.Menu objMenu = null;
            if (IdMenu.Trim().Length > 0)
            {
                strSQL = "SELECT IdMenu,Nom,IdPlat1,IdPlat2,IdPostre,Comensals,IVA,Preu,Beguda FROM Menus WHERE IdMenu = " + objBD.VtoSQL(IdMenu);
                DataSet ds = objBD.GenerarSelect(strSQL);
                if (ds.Tables[0].Rows.Count > 0)
                {
                    objMenu = new ComuRestaurant.Objectes.Menu();
                    objMenu.IdMenu =Convert.ToInt32(ds.Tables[0].Rows[0]["IdMenu"]);
                    objMenu.Nom = ds.Tables[0].Rows[0]["Nom"].ToString();
                    dalcPlats objPlat = new dalcPlats();
                    objMenu.Plat1 = objPlat.Obtindre(ds.Tables[0].Rows[0]["IdPlat1"].ToString());
                    objMenu.Plat2 = objPlat.Obtindre(ds.Tables[0].Rows[0]["IdPlat2"].ToString());
                    objMenu.Postre = objPlat.Obtindre(ds.Tables[0].Rows[0]["IdPostre"].ToString());
                    objMenu.Comensals =Convert.ToInt32(ds.Tables[0].Rows[0]["Comensals"]);
                    objMenu.Preu = ds.Tables[0].Rows[0]["Preu"].ToString();
                    objMenu.IVA =Convert.ToBoolean(ds.Tables[0].Rows[0]["IVA"]);
                    objMenu.Beguda = Convert.ToBoolean(ds.Tables[0].Rows[0]["Beguda"].ToString());            
                }
            }
            return objMenu;
        }

        ///<summary>
        ///Elimina un menu en concret de la base de dades.
        ///</summary>
        ///<return>
        ///retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="IdMenu">
        ///Cadena amb l'identificador de registre d'un menu.
        ///</param>
        public string Eliminar(string IdMenu)
        {
            string strSQL = "DELETE FROM Menus WHERE IdMenu=@IdMenu";
            System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[1];
            sSqlParametros[0] = objBD.CrearParametreSQL("@IdMenu", IdMenu, false);
            string strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);

            return strRet;
        }
        
        ///<summary>
        ///Comprova si existeixen registres amb l'identificador solicitat per parametre.
        ///</summary>
        ///<return>
        ///Retora un boleà indicant si existeix o no un registre en concret a la base de dades.
        ///</return>
        ///<param name="IdMenu">
        ///Identificador de Menu a comprovar si existeix.
        ///</param>
        public bool Existeix(int IdMenu)
        {
            string strSQL = "SELECT IdMenu FROM Menus WHERE IdMenu = " + objBD.VtoSQL(IdMenu);
            DataSet ds = objBD.GenerarSelect(strSQL);
            if (ds.Tables[0].Rows.Count > 0)
            {
                return true;
            }
            return false;
        }


        //<summary>
        ///Obté una llista dels menus que existeixen a la base de dades i informa les seves dades d'un objecte Menu.
        ///</summary>
        ///<return>
        ///Retornat un objecte Menus que conte una llista d'objetes menu amb la informació dels menús.
        ///</return>
        public List<ComuRestaurant.Objectes.Menu> ObtindreTots()
        {
            string strSQL = "";
            ComuRestaurant.Objectes.Menu objMenu = null;
            ComuRestaurant.Objectes.Menus colMenu = new ComuRestaurant.Objectes.Menus();
            dalcPlats objPlat = new dalcPlats();

            strSQL = "SELECT IdMenu,Nom,IdPlat1,IdPlat2,IdPostre,Comensals,IVA,Preu,Beguda FROM Menus ";
            DataSet ds = objBD.GenerarSelect(strSQL);
            for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
            {
                objMenu = new ComuRestaurant.Objectes.Menu();
                objMenu.IdMenu = Convert.ToInt32(ds.Tables[0].Rows[i]["IdMenu"]);
                objMenu.Nom = ds.Tables[0].Rows[i]["Nom"].ToString(); 
                objMenu.Plat1 = objPlat.Obtindre(ds.Tables[0].Rows[i]["IdPlat1"].ToString());
                objMenu.Plat2 = objPlat.Obtindre(ds.Tables[0].Rows[i]["IdPlat2"].ToString());
                objMenu.Postre = objPlat.Obtindre(ds.Tables[0].Rows[i]["IdPostre"].ToString());
                objMenu.Comensals = Convert.ToInt32(ds.Tables[0].Rows[i]["Comensals"]);
                objMenu.Preu = ds.Tables[0].Rows[i]["Preu"].ToString();
                objMenu.Beguda = Convert.ToBoolean(ds.Tables[0].Rows[i]["Beguda"]);
                objMenu.IVA = Convert.ToBoolean(ds.Tables[0].Rows[i]["IVA"]);

                colMenu.addMenu(objMenu);
            }
            return colMenu.getllista(); 
        }

        /// <summary>
        /// Obté el total de menus que hi ha la base de dades.
        /// </summary>
        /// <returns>
        /// Retorna un integer amb el  número total de menús que hi ha a la base de dades.
        /// </returns>
        public int TotalMenus()
        {
            string strSQL = "SELECT Count(*) As Total FROM Menus";
            DataSet ds = objBD.GenerarSelect(strSQL);
            return Convert.ToInt32(ds.Tables[0].Rows[0]["Total"].ToString()); ;
        }
    }
}
