﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using WcfRestaurant.Procesos;


namespace WcfRestaurant.Dalc
{
    public class dalcPlats
    {
        dalcLlibreriaBD objBD;

        ///<summary>
        ///Constructor de l'objecte que gestiona l'acces a dades.
        ///</summary>
        public dalcPlats()
        {
            objBD = new dalcLlibreriaBD();
        }

        ///<summary>
        ///Guarda la informació de l'objecte Plat a la base de dades. Ja sigui insertant un registre nou o actualizat un registre ja existent.
        ///</summary>
        ///<return>
        ///Retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="objPlat">
        ///Objecte Plat que conté la informació que es vol guardar.
        ///</param>
        public string Desar(ComuRestaurant.Objectes.Plat objPlat)
        {
            string strRet = "";
            if (objPlat!=null){
                if (objPlat.NomCAT.Trim().Length > 0)
                {
                    string strSQL = "";
                    bool bExisteix = Existeix(objPlat.NomCAT);

                    strSQL = "SELECT IdTipus FROM TipusPlats WHERE Nom = " + objBD.VtoSQL(objPlat.Tipus.Nom);
                    DataSet ds = objBD.GenerarSelect(strSQL);
                    if (ds.Tables[0].Rows.Count > 0)
                    {
                        objPlat.Tipus.IdTipus = Convert.ToInt32(ds.Tables[0].Rows[0]["IdTipus"]);
                    }

                    System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[7];
                    sSqlParametros[0] = objBD.CrearParametreSQL("@IdPlat", objPlat.IdPlat, false);
                    sSqlParametros[1] = objBD.CrearParametreSQL("@NomCAT", objPlat.NomCAT, false);
                    sSqlParametros[2] = objBD.CrearParametreSQL("@NomESP", objPlat.NomESP, true);
                    sSqlParametros[3] = objBD.CrearParametreSQL("@NomENG", objPlat.NomENG, true);
                    sSqlParametros[4] = objBD.CrearParametreSQL("@IdTipus",objPlat.Tipus.IdTipus, false);
                    sSqlParametros[5] = objBD.CrearParametreSQL("@Preu",objPlat.Preu, false);
                    sSqlParametros[6] = objBD.CrearParametreSQL("@Comensals", objPlat.Comensals, false);
                    if (bExisteix || objPlat.IdPlat!=0){
                        strSQL = "UPDATE Plats SET NomCAT=@NomCAT,NomESP=@NomESP,NomENG=@NomENG,IdTipus=@IdTipus,Comensals=@Comensals,Preu=@Preu WHERE IdPlat=@IdPlat";
                    }else {
                        strSQL = "INSERT INTO Plats(NomCAT,NomESP,NomENG,IdTipus,Comensals,Preu) VALUES (@NomCAT,@NomESP,@NomENG,@IdTipus,@Comensals,@Preu)";
                    }
                    strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);
                }else{
                    strRet = "El Nom del plat no esta informat.";
                }
            }else{
                strRet = "L'objecte del Plat esta vuit.";
            }
            return  strRet;
        }

        ///<summary>
        ///Obté un plat en concret de la base de dades i informa les seves dades en un objecte Platllador.
        ///</summary>
        ///<return>
        ///Retorna un objecte Plat amb les dades del registre amb l'identificador solicitat.
        ///</return>
        ///<param name="IdPlat">
        ///Cadena amb l'identificador de registre d'un plat.
        ///</param>
        public ComuRestaurant.Objectes.Plat Obtindre(string IdPlat)
        {
            string strSQL = "";
            ComuRestaurant.Objectes.Plat objPlat = null;
            if (IdPlat.Trim().Length > 0)
            {
                strSQL = "SELECT IdPlat,NomCAT,NomESP,NomENG,P.IdTipus As IdTipus,Preu,Comensals,Nom FROM Plats P LEFT JOIN TipusPlats TP ON (P.IdTipus=TP.IdTIpus)  WHERE IdPlat = " + objBD.VtoSQL(IdPlat);
                DataSet ds = objBD.GenerarSelect(strSQL);
                if (ds.Tables[0].Rows.Count > 0)
                {
                    objPlat = new ComuRestaurant.Objectes.Plat();
                    objPlat.IdPlat = Convert.ToInt32(ds.Tables[0].Rows[0]["IdPlat"]);
                    objPlat.NomCAT = ds.Tables[0].Rows[0]["NomCAT"].ToString();
                    objPlat.NomESP = ds.Tables[0].Rows[0]["NomESP"].ToString();
                    objPlat.NomENG = ds.Tables[0].Rows[0]["NomENG"].ToString();
                    ComuRestaurant.Objectes.TipusPlat oTipus = new ComuRestaurant.Objectes.TipusPlat();
                    oTipus.IdTipus = System.Convert.ToInt32(ds.Tables[0].Rows[0]["IdTipus"]);
                    oTipus.Nom = ds.Tables[0].Rows[0]["Nom"].ToString();
                    objPlat.Tipus = oTipus;
                    objPlat.Preu = ds.Tables[0].Rows[0]["Preu"].ToString();
                    objPlat.Comensals = Convert.ToInt32(ds.Tables[0].Rows[0]["Comensals"]);
                    
                }
            }
            return objPlat;
        }

        ///<summary>
        ///Elimina un plat en concret de la base de dades.
        ///</summary>
        ///<return>
        ///retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="IdPlat">
        ///Cadena amb l'identificador de registre d'un plat.
        ///</param>
        public string Eliminar(string IdPlat)
        {
            string strSQL = "DELETE FROM Plats WHERE IdPlat=@IdPlat";
            System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[1];
            sSqlParametros[0] = objBD.CrearParametreSQL("@IdPlat", IdPlat, false);
            string strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);

            return strRet;
        }

        ///<summary>
        ///Comprova si existeixen registres amb l'identificador solicitat per parametre.
        ///</summary>
        ///<return>
        ///Retora un boleà indicant si existeix o no un registre en concret a la base de dades.
        ///</return>
        ///<param name="Nom">
        ///Nom del plat a comprovar si existeix.
        ///</param>
        public bool Existeix(string Nom)
        {
            string strSQL = "SELECT IdPlat FROM Plats WHERE NomCAT = " + objBD.VtoSQL(Nom);
            DataSet ds = objBD.GenerarSelect(strSQL);
            if (ds.Tables[0].Rows.Count > 0)
            {
                return true;
            }
            return false;
        }
        
        ///<summary>
        ///Comprova si existeixen registres amb l'identificador solicitat per parametre i retorna el seu identificador.
        ///</summary>
        ///<return>
        ///Retora un int indicant l'identificador del registre en concret a la base de dades.
        ///</return>
        ///<param name="Nom">
        ///Nom del plat a buscar.
        ///</param>
        public int ObtenirIdentificador(string Nom)
        {
            string strSQL = "SELECT IdPlat FROM Plats WHERE NomCAT = " + objBD.VtoSQL(Nom);
            DataSet ds = objBD.GenerarSelect(strSQL);
            if (ds.Tables[0].Rows.Count > 0)
            {
                return Convert.ToInt32(ds.Tables[0].Rows[0]["IdPlat"]);
            }
            return 0;
        }

        ///<summary>
        ///Comprova si existeixen registres amb l'identificador solicitat per parametre i retorna el nom del Plat.
        ///</summary>
        ///<return>
        ///Retora un string indicant el nom del plat del registre en concret a la base de dades.
        ///</return>
        ///<param name="IdPlat">
        ///Identificador del plat a buscar.
        ///<param name="PlatComanda">
        ///Posició del plat dins la comanda
        ///</param>
        public string VerificarPlat(string IdPlat,string PlatComanda)
        {
            string strSQL = "SELECT NomCAT FROM Plats WHERE IdPlat = " + objBD.VtoSQL(IdPlat);
            DataSet ds = objBD.GenerarSelect(strSQL);
            if (ds.Tables[0].Rows.Count > 0)
            {
                return PlatComanda + "-" + ds.Tables[0].Rows[0]["NomCAT"].ToString();
            }
            return PlatComanda + "-Plat Inexistent";
        }

        //<summary>
        ///Obté una llista dels plats que existeixen a la base de dades i informa les seves dades d'un objecte Plat.
        ///</summary>
        ///<return>
        ///Retornat un objecte Plats que conte una llista d'objetes Plat amb la informació dels plats.
        ///</return>
        public List<ComuRestaurant.Objectes.Plat> ObtindreTots()
        {
            string strSQL = "";
            ComuRestaurant.Objectes.Plat objPlat = null;
            ComuRestaurant.Objectes.Plats colPlat = new ComuRestaurant.Objectes.Plats();

            strSQL = "SELECT IdPlat,NomCAT,NomESP,NomENG,P.IdTipus As IdTipus,Preu,Comensals,Nom FROM Plats P LEFT JOIN TipusPlats TP ON (P.IdTipus=TP.IdTIpus) ";
            DataSet ds = objBD.GenerarSelect(strSQL);
            for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
            {
                objPlat = new ComuRestaurant.Objectes.Plat();
                objPlat.IdPlat = Convert.ToInt32(ds.Tables[0].Rows[i]["IdPlat"]);
                objPlat.NomCAT = ds.Tables[0].Rows[i]["NomCAT"].ToString();
                objPlat.NomESP = ds.Tables[0].Rows[i]["NomESP"].ToString();
                objPlat.NomENG = ds.Tables[0].Rows[i]["NomENG"].ToString();
                ComuRestaurant.Objectes.TipusPlat oTipus = new ComuRestaurant.Objectes.TipusPlat();
                oTipus.IdTipus = System.Convert.ToInt32(ds.Tables[0].Rows[i]["IdTipus"]);
                oTipus.Nom = ds.Tables[0].Rows[i]["Nom"].ToString();
                objPlat.Tipus = oTipus;
                objPlat.Preu =ds.Tables[0].Rows[i]["Preu"].ToString();
                objPlat.Comensals = Convert.ToInt32(ds.Tables[0].Rows[i]["Comensals"]);

                colPlat.addPlat(objPlat);
            }
            return colPlat.getllista(); 
        }


        //<summary>
        ///Obté una llista dels plats del tipus indicat que existeixen a la base de dades i informa les seves dades d'un objecte Plat.
        ///</summary>
        ///<return>
        ///Retornat un objecte Plats que conte una llista d'objetes Plat amb la informació dels plats.
        ///</return>
        ///<param name="IdTipus">
        ///Tipus del plat a comprovar si existeix.
        ///</param>
        public List<ComuRestaurant.Objectes.Plat> ObtindreTotsPerTipus(int IdTipus)
        {
            string strSQL = "";
            ComuRestaurant.Objectes.Plat objPlat = null;
            ComuRestaurant.Objectes.Plats colPlat = new ComuRestaurant.Objectes.Plats();

            strSQL = "SELECT IdPlat,NomCAT,NomESP,NomENG,P.IdTipus As IdTipus,Preu,Comensals,Nom FROM Plats P LEFT JOIN TipusPlats TP ON (P.IdTipus=TP.IdTIpus) WHERE P.IdTipus=" + objBD.VtoSQL(IdTipus);
            DataSet ds = objBD.GenerarSelect(strSQL);
            for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
            {
                objPlat = new ComuRestaurant.Objectes.Plat();
                objPlat.IdPlat = Convert.ToInt32(ds.Tables[0].Rows[i]["IdPlat"]);
                objPlat.NomCAT = ds.Tables[0].Rows[i]["NomCAT"].ToString();
                objPlat.NomESP = ds.Tables[0].Rows[i]["NomESP"].ToString();
                objPlat.NomENG = ds.Tables[0].Rows[i]["NomENG"].ToString();
                ComuRestaurant.Objectes.TipusPlat oTipus = new ComuRestaurant.Objectes.TipusPlat();
                oTipus.IdTipus = System.Convert.ToInt32(ds.Tables[0].Rows[i]["IdTipus"]);
                oTipus.Nom = ds.Tables[0].Rows[i]["Nom"].ToString();
                objPlat.Tipus = oTipus;
                objPlat.Preu = ds.Tables[0].Rows[i]["Preu"].ToString();
                objPlat.Comensals = Convert.ToInt32(ds.Tables[0].Rows[i]["Comensals"]);

                colPlat.addPlat(objPlat);
            }
            return colPlat.getllista();
        }

        /// <summary>
        /// Obté el total de plats que hi ha la base de dades.
        /// </summary>
        /// <returns>
        /// Retorna un intege ramb el  número total de plats que hi ha a la base de dades.
        /// </returns>
        public int TotalPlats()
        {
            string strSQL = "SELECT Count(*) FROM Plats ";
            DataSet ds = objBD.GenerarSelect(strSQL);
            return ds.Tables[0].Rows.Count ;
        }
    }
}
