﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using WcfRestaurant.Procesos;

namespace WcfRestaurant.Dalc
{
    class dalcTipusPlats
    {
        dalcLlibreriaBD objBD;

        ///<summary>
        ///Constructor de l'objecte que gestiona l'acces a dades.
        ///</summary>
        public dalcTipusPlats()
        {
            objBD = new dalcLlibreriaBD();
        }



        ///<summary>
        ///Obté un tipus de plat en concret de la base de dades i informa les seves dades en un objecte TIpusPlat.
        ///</summary>
        ///<return>
        ///Retorna un objecte TipusPlat amb les dades del registre amb l'identificador solicitat.
        ///</return>
        ///<param name="IdTipusPlat">
        ///Cadena amb l'identificador de registre d'un tipus de plat.
        ///</param>
        public ComuRestaurant.Objectes.TipusPlat Obtindre(string IdTipusPlat)
        {
            string strSQL = "";
            ComuRestaurant.Objectes.TipusPlat objTipus = null;
            if (IdTipusPlat.Trim().Length > 0)
            {
                strSQL = "SELECT IdTipus,Nom FROM TipusPlats WHERE IdTIpus = " + objBD.VtoSQL(IdTipusPlat);
                DataSet ds = objBD.GenerarSelect(strSQL);
                if (ds.Tables[0].Rows.Count > 0)
                {
                    objTipus = new ComuRestaurant.Objectes.TipusPlat();
                    objTipus.IdTipus = Convert.ToInt32(ds.Tables[0].Rows[0]["IdTipus"]);
                    objTipus.Nom = ds.Tables[0].Rows[0]["Nom"].ToString();
                }
            }
            return objTipus;
        }


        ///<summary>
        ///Elimina un Tipus de Plat en concret de la base de dades.
        ///</summary>
        ///<return>
        ///retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="IdTipus">
        ///Cadena amb l'identificador de registre d'un tipus de plat.
        ///</param>
        public string Eliminar(string IdTipus)
        {
            string strSQL = "DELETE FROM TIpusPlats WHERE IdTipus=@IdTipus";
            System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[1];
            sSqlParametros[0] = objBD.CrearParametreSQL("@IdTipus", IdTipus, false);
            string strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);

            return strRet;
        }
        

        //<summary>
        ///Obté una llista dels tipus de plats que existeixen a la base de dades i informa les seves dades d'un objecte TIpusPlat.
        ///</summary>
        ///<return>
        ///Retorna un objecte TIpusPlats que conte una llista d'objetes TipusPlat amb la informació dels tipus de plats.
        ///</return>
        public List<ComuRestaurant.Objectes.TipusPlat> ObtindreTots()
        {
            string strSQL = "";
            ComuRestaurant.Objectes.TipusPlat objTipus = null;
            ComuRestaurant.Objectes.TipusPlats colTipus = new ComuRestaurant.Objectes.TipusPlats();

            strSQL = "SELECT IdTipus,Nom FROM TIpusPlats";
            DataSet ds = objBD.GenerarSelect(strSQL);
            for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
            {
                objTipus = new ComuRestaurant.Objectes.TipusPlat();
                objTipus.IdTipus = Convert.ToInt32(ds.Tables[0].Rows[i]["IdTipus"]);
                objTipus.Nom = ds.Tables[0].Rows[i]["Nom"].ToString();
                colTipus.addTipusPlat(objTipus);
            }
            return colTipus.getllista(); 
        }

        /// <summary>
        /// Obté el total de tipus de plats que hi ha la base de dades.
        /// </summary>
        /// <returns>
        /// Retorna un integer amb el número total de tipus de plats que hi ha a la base de dades.
        /// </returns>
        public int TotalTipusPlats()
        {
            string strSQL = "SELECT Count(IdTipus) FROM TipusPlats";
            DataSet ds = objBD.GenerarSelect(strSQL);
            return ds.Tables[0].Rows.Count ;
        }

        ///<summary>
        ///Guarda la informació de l'objecte TipusPlat a la base de dades. Ja sigui insertant un registre nou o actualizat un registre ja existent.
        ///</summary>
        ///<return>
        ///Retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="objTipusPlat">
        ///Objecte TIpusPlat que conté la informació que es vol guardar.
        ///</param>
        public string Desar(ComuRestaurant.Objectes.TipusPlat objTipusPlat)
        {
            string strRet = "";
            if (objTipusPlat != null)
            {
                if (objTipusPlat.Nom.Trim().Length > 0)
                {
                    string strSQL = "";
                    bool bExisteix = Existeix(objTipusPlat.Nom);

                    System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[2];
                    sSqlParametros[0] = objBD.CrearParametreSQL("@Nom", objTipusPlat.Nom, false);
                    sSqlParametros[1] = objBD.CrearParametreSQL("@IdTipus", objTipusPlat.IdTipus, false);

                    if (bExisteix)
                    {
                        strRet = "El nom del tipus ja existeix.";
                    }
                    else
                    {
                        if (objTipusPlat.IdTipus == 0)
                        {
                            strSQL = "INSERT INTO TipusPlats (Nom) VALUES (@Nom)";
                        }
                        else
                        {
                            strSQL = "UPDATE TipusPlats SET Nom = @Nom WHERE IdTipus=@IdTipus";
                        }
                        strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);
                    }   
                }
                else
                {
                    strRet = "El nom del tipus no esta informat.";
                }
            }
            else
            {
                strRet = "L'objecte del tipus de plat esta vuit.";
            }
            return strRet;
        }

        ///<summary>
        ///Comprova si existeixen registres amb el nom solicitat per paràmetre.
        ///</summary>
        ///<return>
        ///Retora un boleà indicant si existeix o no un registre en concret a la base de dades.
        ///</return>
        ///<param name="Nom">
        ///Nom del tipus de plat a comprovar si existeix.
        ///</param>
        public bool Existeix(string Nom)
        {
            string strSQL = "SELECT * FROM TipusPlats WHERE Nom = " + objBD.VtoSQL(Nom);
            DataSet ds = objBD.GenerarSelect(strSQL);
            if (ds.Tables[0].Rows.Count > 0)
            {
                return true;
            }
            return false;
        }
    }


}
