﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using WcfRestaurant.Procesos;


namespace WcfRestaurant.Dalc
{
    public class dalcTreballador
    {
        dalcLlibreriaBD objBD;

        ///<summary>
        ///Constructor de l'objecte que gestiona l'acces a dades.
        ///</summary>
        public dalcTreballador()
        {
            objBD = new dalcLlibreriaBD();
        }

        ///<summary>
        ///Guarda la informació de l'objecte treballador a la base de dades. Ja sigui insertant un registre nou o actualizat un registre ja existent.
        ///</summary>
        ///<return>
        ///Retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="objTreballador">
        ///Objecte treballador que conté la informació que es vol guardar.
        ///</param>
        public string Desar(ComuRestaurant.Objectes.Treballador objTreballador)
        {
            string strRet = "";
            if (objTreballador!=null){
                if (objTreballador.IdTreballador.Trim().Length > 0)
                {
                    string strSQL = "";
                    bool bExisteix = Existeix(objTreballador.IdTreballador);

                    System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[5];
                    sSqlParametros[0] = objBD.CrearParametreSQL("@IdTreballador", objTreballador.IdTreballador, false);
                    sSqlParametros[1] = objBD.CrearParametreSQL("@Nom", objTreballador.Nom, false);
                    sSqlParametros[2] = objBD.CrearParametreSQL("@cognom1", objTreballador.Cognom1, true);
                    sSqlParametros[3] = objBD.CrearParametreSQL("@cognom2", objTreballador.Cognom2, true);
                    sSqlParametros[4] = objBD.CrearParametreSQL("@rol",objTreballador.IdRol.IdRol, false);
                    if (bExisteix){
                        strSQL = "UPDATE Treballadors SET Nom = @Nom,Cognom1=@cognom1,Cognom2=@cognom2,IdRol=@rol WHERE IdTreballador=@IdTreballador";
                    }else {
                        strSQL = "INSERT INTO Treballadors (IdTreballador,Nom,Cognom1,Cognom2,IdRol,Contrasenya) VALUES (@IdTreballador,@Nom,@cognom1,@cognom2,@rol,@IdTreballador)";
                    }
                    strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);
                }else{
                    strRet = "L'identificador del objecte d'usuari no esta informat.";
                }
            }else{
                strRet = "L'objecte del treballador esta vuit.";
            }
            return  strRet;
        }

        ///<summary>
        ///Obté un treballador en concret de la base de dades i informa les seves dades en un objecte treballador.
        ///</summary>
        ///<return>
        ///Rttorna un objecte treballador amb les dades del registre amb l'identificador solicitat.
        ///</return>
        ///<param name="IdTreballador">
        ///Cadena amb l'identificador de registre d'un treballador.
        ///</param>
        public ComuRestaurant.Objectes.Treballador Obtindre(string IdTreballador)
        {
            string strSQL = "";
            ComuRestaurant.Objectes.Treballador objTreballador = null;
            if (IdTreballador.Trim().Length > 0)
            {
                strSQL = "SELECT IdTreballador,Nom,Cognom1,Cognom2,IdRol FROM Treballadors WHERE IdTreballador = " + objBD.VtoSQL(IdTreballador);
                DataSet ds = objBD.GenerarSelect(strSQL);
                if (ds.Tables[0].Rows.Count > 0)
                {
                    objTreballador = new ComuRestaurant.Objectes.Treballador();
                    objTreballador.IdTreballador = ds.Tables[0].Rows[0]["IdTreballador"].ToString();
                    objTreballador.Nom = ds.Tables[0].Rows[0]["Nom"].ToString();
                    objTreballador.Cognom1 = ds.Tables[0].Rows[0]["Cognom1"].ToString();
                    objTreballador.Cognom2 = ds.Tables[0].Rows[0]["Cognom2"].ToString();
                    ComuRestaurant.Objectes.Rol oRol = new ComuRestaurant.Objectes.Rol();
                    oRol.IdRol = System.Convert.ToInt32(ds.Tables[0].Rows[0]["IdRol"]);
                    objTreballador.IdRol = oRol;
                }
            }
            return objTreballador;
        }

        ///<summary>
        ///Elimina un treballador en concret de la base de dades.
        ///</summary>
        ///<return>
        ///retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="IdTreballador">
        ///Cadena amb l'identificador de registre d'un treballador.
        ///</param>
        public string Eliminar(string IdTreballador)
        {
            string strSQL = "DELETE FROM Treballadors WHERE IdTreballador=@IdTreballador";
            System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[1];
            sSqlParametros[0] = objBD.CrearParametreSQL("@IdTreballador", IdTreballador, false);
            string strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);

            return strRet;
        }
        
        ///<summary>
        ///Comprova si existeixen registres amb l'identificador solicitat per parametre.
        ///</summary>
        ///<return>
        ///Retora un boleà indicant si existeix o no un registre en concret a la base de dades.
        ///</return>
        ///<param name="IdTreballador">
        ///Identificador d'usuari a comprovar si existeix.
        ///</param>
        public bool Existeix(string IdTreballador)
        {
            string strSQL = "SELECT IdTreballador FROM Treballadors WHERE IdTreballador = " + objBD.VtoSQL(IdTreballador);
            DataSet ds = objBD.GenerarSelect(strSQL);
            if (ds.Tables[0].Rows.Count > 0)
            {
                return true;
            }
            return false;
        }

        ///<summary>
        ///Comprova si l'usuario i la contrasenya son correctes a la base de ddades.
        ///</summary>
        ///<return>
        ///Retorna el rol del treballador.
        ///</return>
        ///<param name="IdTreballador">
        ///Identificador del treballador a comprovar si existeix.
        ///</param>
        ///<param name="strContrasenya">
        ///Contrasenya a comprovar a la base de dades pel treballador del primer parametre.
        ///</param>
        public int ComprovarContrasenya(string IdTreballador, string strContrasenya)
        {
            string strSQL = "SELECT IdRol FROM Treballadors WHERE IdTreballador = " + objBD.VtoSQL(IdTreballador);
            strSQL += " AND contrasenya = " + objBD.VtoSQL(strContrasenya);
            DataSet ds = objBD.GenerarSelect(strSQL);
            if (ds.Tables[0].Rows.Count == 0)
            {
                return 0;
            }
            return System.Convert.ToInt32(ds.Tables[0].Rows[0]["IdRol"]); ;
        }

        ///<summary>
        ///Assigna una contrasenya a un usuari.
        ///</summary>
        ///<return>
        ///Retorna una cadena de texte informant dels errors esdevenits. Si la cadena esta vuida tot a funcionat correctament.
        ///</return>
        ///<param name="IdTreballador">
        ///Cadena amb l'identificador de registre d'un usuari.
        ///</param>
        ///<param name="strContrasenya">
        ///Contrasenya a assignar en la base de dades per l'usuari del primer parametre.
        ///</param>
        public string AssignarContrasenya(string IdTreballador, string strContrasenya)
        {
            string strRet ="";
            if (IdTreballador.Trim().Length > 0) 
            {
                string strSQL = "UPDATE Treballadors SET Contrasenya=@contrasenya WHERE IdTreballador=@IdTreballador";
                System.Data.SqlClient.SqlParameter[] sSqlParametros = new System.Data.SqlClient.SqlParameter[2];
                sSqlParametros[0] = objBD.CrearParametreSQL("@IdTreballador", IdTreballador, false);
                sSqlParametros[1] = objBD.CrearParametreSQL("@contrasenya", strContrasenya, false);
                strRet = objBD.ExecutarSentencia(strSQL, sSqlParametros);
            }
            else
            {
                strRet = "L'identificador de l'objecte d'usuari no esta informat.";
            }
            return strRet;
        }

        //<summary>
        ///Obté una llista dels treballadors que existeixen a la base de dades i informa les seves dades d'un objecte Treballador.
        ///</summary>
        ///<return>
        ///Retornat un objecte Treballadors que conte una llista d'objetes trebalador amb la informació dels treballadors.
        ///</return>
        public List<ComuRestaurant.Objectes.Treballador> ObtindreTots()
        {
            string strSQL = "";
            ComuRestaurant.Objectes.Treballador objTreballador = null;
            ComuRestaurant.Objectes.Treballadors colTreballador = new ComuRestaurant.Objectes.Treballadors();

            strSQL = "SELECT IdTreballador,T.Nom As Nom,Cognom1,Cognom2,T.IdRol,R.Nom As RNOM,Contrasenya FROM Treballadors T LEFT JOIN Rols R ON T.IdRol=R.IdRol ";
            DataSet ds = objBD.GenerarSelect(strSQL);
            for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
            {
                objTreballador = new ComuRestaurant.Objectes.Treballador();
                objTreballador.IdTreballador = ds.Tables[0].Rows[i]["IdTreballador"].ToString();
                objTreballador.Nom = ds.Tables[0].Rows[i]["Nom"].ToString();
                objTreballador.Cognom1 = ds.Tables[0].Rows[i]["Cognom1"].ToString();
                objTreballador.Cognom2 = ds.Tables[0].Rows[i]["Cognom2"].ToString();
                objTreballador.Contrasenya = ds.Tables[0].Rows[i]["Contrasenya"].ToString();
                ComuRestaurant.Objectes.Rol oRol = new ComuRestaurant.Objectes.Rol();
                oRol.IdRol = System.Convert.ToInt32(ds.Tables[0].Rows[i]["IdRol"]);
                oRol.Nombre = ds.Tables[0].Rows[i]["RNOM"].ToString();
                objTreballador.IdRol = oRol;

                colTreballador.addTreballador(objTreballador);
            }
            return colTreballador.getllista(); 
        }

        /// <summary>
        /// Obté el total de treballadors que hi ha la base de dades.
        /// </summary>
        /// <returns>
        /// Retorna un integeramb el  número total de treballadors que hi ha a la base de dades.
        /// </returns>
        public int TotalTreballadors()
        {
            string strSQL = "SELECT Count(*) FROM Treballadors ";
            DataSet ds = objBD.GenerarSelect(strSQL);
            return ds.Tables[0].Rows.Count ;
        }
    }
}
