﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WcfRestaurant.Procesos
{
    public class tfcFuncions
    {
        ///<summary>
        ///Convierte una cadena de texto en un formato DateTime.
        ///</summary>
        ///<return>
        ///Devuelve una fecha DateTime o un valor nulo si el valor no es convertible a DateTime.
        ///</return>
        ///<param name="strFecha">
        ///Fecha en formato cadena a ser convertida en DateTime.
        ///</param> 
        public static DateTime? ConvAFecha(string strFecha)
        {
            DateTime? dtFecha = null;
            try
            {
                dtFecha = Convert.ToDateTime(strFecha);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return dtFecha;
        }

        ///<summary>
        ///Convierte una cadena de texto en un formato Entero.
        ///</summary>
        ///<return>
        ///Devuelve un valor entero o un valor 0 si el valor no es convertible a entero.
        ///</return>
        ///<param name="strEntero">
        ///Valor entero en formato cadena a ser convertida en un valor entero.
        ///</param>
        public static int ConvAEntero(string strEntero)
        {
            int intEntero = -1;
            bool bRet;
            try
            {
                bRet = Int32.TryParse(strEntero, out intEntero);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return intEntero;
        }

        ///<summary>
        ///Convierte un valor entero en una cadena de texto en formato hora:minutos.
        ///</summary>
        ///<return>
        ///Devuelve un valor cadena con la conversion del valor entero pasado a formato hora:minutos.
        ///</return>
        ///<param name="intEntero">
        ///Valor entero a ser convertido en una cadena con formato hora:minutos.
        ///</param>
        public static string ConvAHoraMinutos(int intEntero)
        {
            string strValor = "";
            try
            {
                int parteEntera = intEntero / 60;
                int parteResiduo = intEntero % 60;

                if (parteEntera < 10) strValor = "0";
                strValor += parteEntera.ToString();
                strValor += ":";
                if (parteResiduo < 10) strValor += "0";
                strValor += parteResiduo.ToString();
            }
            catch (Exception ex)
            {
                strValor = "00:00";
                Console.WriteLine(ex.Message.ToString());
            }
            return strValor;
        }

        ///<summary>
        ///Convierte un valor cadena con formato hora:minutos en un entero que representan los minutos del dia.
        ///</summary>
        ///<return>
        ///Devuelve un valor entero con la conversión a minutos de la cadena mandada por parametro.
        ///</return>
        ///<param name="strHora">
        ///Valor cadena en formato hora:minutos a ser convertido en un entero que represente los minutos del dia
        ///</param>
        public static int ConvAHoraMinutos(string strHora)
        {
            int intValor = 0;
            try
            {
                if (strHora.IndexOf(':') > 0)
                {
                    string[] strPartes = strHora.Split(':');
                    int intHora = ConvAEntero(strPartes[0]);
                    int intMin = ConvAEntero(strPartes[1]);

                    intValor = (intHora * 60) + intMin;
                }
            }
            catch (Exception ex)
            {
                intValor = 0;
                Console.WriteLine(ex.Message.ToString());
            }
            return intValor;
        }

        /// <summary>
        /// Genera una contraseña formada por una cadena aleatoria de letras con el tamaño pasado por parametro
        /// </summary>
        /// <returns>
        /// Cadena de texto aleatorio
        /// </returns>
        /// <param name="intTamano">
        /// Tamaño de la cadena que se devolverá
        /// </param>
        /// <param name="bMinusculas">
        /// Si se pasa un valor verdadero devuelve el valor en minusculas
        /// </param>
        public static string GetContrasena(int intTamano, bool bMinusculas)
        {
            StringBuilder strBuild = new StringBuilder();
            Random aleatorio = new Random();
            char ch;
            for (int i = 0; i < intTamano; i++)
            {
                ch = Convert.ToChar(Convert.ToInt32(Math.Floor(26 * aleatorio.NextDouble() + 65)));
                strBuild.Append(ch);
            }
            if (bMinusculas)
                return strBuild.ToString().ToLower();
            return strBuild.ToString();
        }


    }
}
